==> crossroads-0.11.tar.gz: crossroads-0.11/src/tcpserve.c <==
#include "crossroads.h"

void tcpserve (int server_sock) {
    fd_set set;
    int backend_sock, new, size, i, pid;
    struct sockaddr_in clientname;

    /* Set up our signal handlers. */
    if (!flag_foreground) {
	/* Interruption signals */
	for (i = 0; relevant_sigs[i]; i++)
	    signal (relevant_sigs[i], interrupt);
	/* Set wakeup handler for the wakeup calls. */
	if (activeservice->rev_interval) {
	    if ( (pid = fork()) < 0 )
		error ("Fork failed: %s", strerror(errno));
	    else if (!pid) {
		set_program_title ("crossroads: wakeup for %s",
				   activeservice->name);
		wakeup_handler();
	    } else {
		msg ("Started wakeup handler at pid %d", pid);
		servicereport->rev_pid = pid;
	    }
	}
    }
    
    msg ("Awaiting activity on port %d (socket fd %d)",
	 activeservice->port, server_sock);

    if (listen (server_sock, activeservice->backlog + 1) < 0)
	error ("Failed to listen to server_socket: %s", strerror(errno));

    /* Avoid zombies. */
    signal (SIGCHLD, SIG_IGN);

    /* We're a service now. Never return, never exit. */
    while (1) {
	/* Block until we get a connection. */
	FD_ZERO (&set);
	FD_SET (server_sock, &set);
	if (select (FD_SETSIZE, &set, 0, 0, 0) < 0)
	    error ("Error while waiting for activity: %d (%s)",
		   errno, strerror(errno));

	/* Accept the client-side connection. */
	size = sizeof(clientname);
	if ( (new = accept (server_sock, (struct sockaddr *) &clientname,
			    (socklen_t *) &size)) < 0 )
	    error ("Failure while accepting on server socket: %s",
		   strerror(errno));

	msg ("Connect on service %s from %s", 
	     activeservice->name,
	     inet_ntoa (clientname.sin_addr));
	
	/* Leave it alone if there are no back ends or if we exceed
	 * the max allowed clients. IN THAT CASE WE RETURN so that
	 * runservice() may close the listener socket, sleep, and
	 * create a new one.
	 */
	if (! backend_available()) {
	    warning ("No back ends available");
	    close (new);
	    return;
	}
	if (activeservice->maxclients &&
	    servicereport->nclients >= activeservice->maxclients) {
	    warning ("Max clients %d exceeded, taking a nap",
		     activeservice->maxclients);
	    close (new);
	    return;
	}

	/* Retry back ends until we succeed. */
	while (1) {
	    choose_backend();
	    if (current_backend < 0) {
		/* No back ends available now. NOTE: we return so that
		 * runservice() will retry in some time. */
		close (new);
		return;
	    }
	    
	    /* Show what's happening. */
	    msg ("Trying back end %s, port %d",
		 activeservice->backend[current_backend].server,
		 activeservice->backend[current_backend].port);
	    
	    /* Connect to the backend. */
	    if ( (backend_sock = backend_connect(0)) < 0 ) {
		lock_reporter();
		servicereport->backendstate[current_backend].failures++;
		servicereport->backendstate[current_backend].avail =
		    st_unavailable;
		unlock_reporter();
		sysrun (activeservice->backend[current_backend].onfailure);
		warning ("Failed to connect to backend %s, port %d",
			 activeservice->backend[current_backend].server,
			 activeservice->backend[current_backend].port);
		continue;
	    }

	    /* Got a live one! */
	    servesockets (new, backend_sock);
	    close (new);
	    close (backend_sock);
	    break;
	}
    }
}

==> crossroads-0.17.tar.gz: crossroads-0.17/src/tcpserve.c <==
#include "crossroads.h"

void tcpserve (int server_sock) {
    fd_set set;
    int backend_sock, new, size, i, pid;
    struct sockaddr_in clientname;
    static int wakeup_started = 0;

    /* Set up our signal handlers. */
    if (!flag_foreground && !wakeup_started++) {
	/* Interruption signals */
	for (i = 0; relevant_sigs[i]; i++)
	    signal (relevant_sigs[i], interrupt);
	/* Set wakeup handler for the wakeup calls. */
	if (activeservice->rev_interval) {
	    if ( (pid = fork()) < 0 )
		error ("Fork failed: %s", strerror(errno));
	    else if (!pid) {
		set_program_title ("crossroads: wakeup for %s",
				   activeservice->name);
		wakeup_handler();
	    } else {
		msg ("Started wakeup handler at pid %d", pid);
		servicereport->rev_pid = pid;
	    }
	}
    }
    
    msg ("Awaiting activity on port %d (socket fd %d)",
	 activeservice->port, server_sock);

    if (listen (server_sock, activeservice->backlog + 1) < 0)
	error ("Failed to listen to server_socket: %s", strerror(errno));

    /* Avoid zombies. */
    signal (SIGCHLD, SIG_IGN);

    /* We're a service now. Never return, never exit
     * (unless something is wrong).
     */
    while (1) {
	/* Block until we get a connection. */
	FD_ZERO (&set);
	FD_SET (server_sock, &set);
	if (select (FD_SETSIZE, &set, 0, 0, 0) < 0)
	    error ("Error while waiting for activity: %d (%s)",
		   errno, strerror(errno));

	/* Accept the client-side connection. */
	size = sizeof(clientname);
	if ( (new = accept (server_sock, (struct sockaddr *) &clientname,
			    (socklen_t *) &size)) < 0 )
	    error ("Failure while accepting on server socket: %s",
		   strerror(errno));

	msg ("Connect on service %s from %s", 
	     activeservice->name,
	     inet_ntoa (clientname.sin_addr));
	
	/* Leave it alone if there are no back ends or if we exceed
	 * the max allowed clients. IN THAT CASE WE RETURN so that
	 * runservice() may close the listener socket, sleep, and
	 * create a new one.
	 */
	if (! backend_available()) {
	    warning ("No back ends available");
	    close (new);
	    return;
	}
	if (activeservice->maxclients &&
	    servicereport->nclients >= activeservice->maxclients) {
	    warning ("Max clients %d exceeded",
		     activeservice->maxclients);
	    close (new);
	    return;
	}

	/* Retry back ends until we succeed. */
	while (1) {
	    choose_backend();
	    if (current_backend < 0) {
		/* No back ends available now. NOTE: we return so that
		 * runservice() will retry in some time. */
		close (new);
		return;
	    }
	    
	    /* Show what's happening. */
	    msg ("Trying back end %s, port %d",
		 activeservice->backend[current_backend].server,
		 activeservice->backend[current_backend].port);
	    
	    /* Connect to the backend. */
	    if ( (backend_sock = backend_connect(0)) < 0 ) {
		lock_reporter();
		servicereport->backendstate[current_backend].failures++;
		servicereport->backendstate[current_backend].avail =
		    st_unavailable;
		unlock_reporter();
		sysrun (activeservice->backend[current_backend].onfailure);
		warning ("Failed to connect to backend %s, port %d",
			 activeservice->backend[current_backend].server,
			 activeservice->backend[current_backend].port);
		continue;
	    }

	    /* Got a live one! */
	    servesockets (new, backend_sock);
	    close (new);
	    close (backend_sock);
	    break;
	}
    }
}
